""" Generates synthetically a single week of workload from an HPC platform. 

We do not fit a model on the data. 

This script just uses the empirical distribution to generate data.

Copyright (C) 2024 Manuel G. Marciani
BSC-CNS Earth Sciences Deparment
"""

# to read csv 
import pandas as pd
# to generate empirical distribution
import scipy
import argparse 
# to manage the arrays confortably 
import numpy as np
   
def arg_parse() -> argparse.ArgumentParser:
    """
    Parse arguments from the command line and check correctness.
    Folder structure will be created if not present 
    :returns: argparse object with the parameters
    """
    parser = argparse.ArgumentParser(description="Build a synthetic workload from an anonymized trace")
    parser.add_argument("opendp_release", type=str, help="path of the anonymized release")
    parser.add_argument("out_dir", type=str, help="relative or absolute path to the output directory")
    parser.add_argument("num_jobs", type=int, help="number of jobs to generate")

    arguments = parser.parse_args()

    return arguments
 
if __name__ == "__main__":

    args = arg_parse()

    # open the synthetic output file
    synth_out = open(f"{args.out_dir}/out.csv", "w")
    synth_out.write("gid,hour,day,alloc_cpus,runtime\n") 

    # read distribution of all jobs
    per_gid_jobs = pd.read_csv(f"{args.opendp_release}/variant/per_gid_num_jobs.csv")
    np.where(per_gid_jobs.num_jobs < 0, 0, per_gid_jobs.num_jobs)
    per_gid_jobs["percentage"] = per_gid_jobs.num_jobs/per_gid_jobs.num_jobs.sum()
    
    distr_per_gid_jobs = scipy.stats.rv_discrete(values=(per_gid_jobs.gid,per_gid_jobs.percentage))

    # generate num_jobs
    for _ in range(args.num_jobs):
        gid = distr_per_gid_jobs.rvs()

        # from the drown gid, open the runtime and alloc cpus 
        gid_runtime = pd.read_csv(f"{args.opendp_release}/variant/{gid}_runtime.csv")
        gid_alloc_cpus = pd.read_csv(f"{args.opendp_release}/variant/{gid}_alloc_cpus.csv")
        gid_submit_hour = pd.read_csv(f"{args.opendp_release}/variant/{gid}_hour_day.csv")
        gid_submit_day = pd.read_csv(f"{args.opendp_release}/variant/{gid}_week_day.csv")

        # TODO remove this whenever I update with data clamping
        gid_runtime.num_jobs = np.where(gid_runtime.num_jobs < 0, 0, gid_runtime.num_jobs)
        gid_alloc_cpus.num_jobs = np.where(gid_alloc_cpus.num_jobs < 0, 0, gid_alloc_cpus.num_jobs)
        gid_submit_hour.num_jobs = np.where(gid_submit_hour.num_jobs < 0, 0, gid_submit_hour.num_jobs)
        gid_submit_day.num_jobs = np.where(gid_submit_day.num_jobs < 0, 0, gid_submit_day.num_jobs)

        # TODO remove this whenever I update the other script to percentages 
        gid_runtime["percentage"] = gid_runtime.num_jobs/gid_runtime.num_jobs.sum()
        gid_alloc_cpus["percentage"] = gid_alloc_cpus.num_jobs/gid_alloc_cpus.num_jobs.sum()
        gid_submit_hour["percentage"] = gid_submit_hour.num_jobs/gid_submit_hour.num_jobs.sum()
        gid_submit_day["percentage"] = gid_submit_day.num_jobs/gid_submit_day.num_jobs.sum()

        distr_runtime = scipy.stats.rv_discrete(values=(gid_runtime.runtime,gid_runtime.percentage))
        distr_alloc_cpus = scipy.stats.rv_discrete(values=(gid_alloc_cpus.alloc_cpus,gid_alloc_cpus.percentage))
        distr_submit_hour = scipy.stats.rv_discrete(values=(gid_submit_hour.hour_day,gid_submit_hour.percentage))
        distr_submit_day = scipy.stats.rv_discrete(values=(gid_submit_day.day_week,gid_submit_day.percentage))

        runtime = distr_runtime.rvs()
        alloc_cpus = distr_alloc_cpus.rvs()
        submit_hour = distr_submit_hour.rvs()
        submit_day = distr_submit_day.rvs()

        synth_out.write(f"{gid},{submit_hour},{submit_day},{alloc_cpus},{runtime}\n")

    synth_out.close()
